// modified test + my custom PIDs by miguelos6
// based purely on Elmduino library
// for WEMOS LOLIN32 OLED
// first read then display (every second)
// test on 2.0.12 ELMduino library

#include "BluetoothSerial.h"
#include "ELMduino.h"
#include "SSD1306Wire.h"        // legacy: #include "SSD1306.h"
#include "WiFi.h"
#include "images.h"

SSD1306Wire display(0x3c, 5, 4);   // ADDRESS, SDA, SCL  -  SDA and SCL usually populate automatically based on your board's pins_arduino.h
//SSD1306Wire display(0x3c, 21, 22);   // ADDRESS, SDA, SCL  -  for external display


#define ELM_PORT SerialBT
#define ESP_BLUETOOTH_NAME "ESP32"


BluetoothSerial SerialBT;
ELM327 myELM327;


void setup()
{
  delay(100);
  Serial.begin(115200);
  ELM_PORT.begin(ESP_BLUETOOTH_NAME, true);

  Serial.println("Attempting to connect to ELM327...");

  // Initialising the UI will init the display too.
  display.init();
  display.setFont(ArialMT_Plain_16);
  // display logo
  display.drawXbm(28, 1, Logo_width, Logo_height, Logo_bits);
  display.display();
  //delay(200);

  // change "V-LINK" to your OBD reader bluetooth name //
  if (!ELM_PORT.connect("V-LINK"))
  {
    Serial.println("Couldn't connect to OBD scanner - Phase 1");
      display.clear();
      display.drawString(0, 6, "OBD error 1");
      display.display();
    delay(3000);
    // let it not connect & retest connection at the end of the loop
    //while (1);
  }

  if (!myELM327.begin(ELM_PORT))
  {
    Serial.println("Couldn't connect to OBD scanner - Phase 2");
      display.clear();
      display.drawString(0, 6, "OBD error 2");
      display.display();  
    delay(3000);
    //while (1);
  }
  
  Serial.println("Connected to ELM327");

}


void loop()
{
  // initialize 
  int32_t rpm = -1;
  int32_t soot = -1;
  int32_t burn = -1;
  int32_t km = -1;
  int32_t litersLeft = -1;

// read values first, then display

// whole magic happens here
// need to query specific PID
// all converted to decimal values :
// 0x22 - Ford/GM -> 34
// 0x20F2 - DPF burn status -> 8434
// 0x336A - DPF percent -> 13162
// 0x3039 - DPF km -> 12345
// 0x132A - liters left -> 4906


// read RPM
  float tempRPM = myELM327.rpm();
  if (myELM327.status == ELM_SUCCESS)
  {
    rpm = (int32_t)tempRPM;
    Serial.print("RPM: "); Serial.println(rpm);
  }
  else
      printError();

// read DPF soot level
  if (myELM327.queryPID(34, 13162))
  {
    int32_t tempSoot = myELM327.findResponse();
    
    if (myELM327.status == ELM_SUCCESS)
    {
      soot = tempSoot;
      Serial.print("DPF soot: "); Serial.println(soot);
    }
    else
      printError();
  }

// read DPF km since last regen
  if (myELM327.queryPID(34, 12345))
  {
    int32_t tempKm = myELM327.findResponse();    

    if (myELM327.status == ELM_SUCCESS)
    {
      km = tempKm;
      Serial.print("DPF km: "); Serial.println(km);
    }
    else 
      printError();
  }

// read fuel liters left 
  if (myELM327.queryPID(34, 4906))
  {
    int32_t tempLitersLeft = myELM327.findResponse() / 64.0;    

    if (myELM327.status == ELM_SUCCESS)
    {
      litersLeft = tempLitersLeft;
      Serial.print("Liters: "); Serial.println(litersLeft);
    }
    else
      printError();
  }

// read DPF burn status 0/1
  if (myELM327.queryPID(34, 8434))
  {
    int32_t tempBurn = myELM327.findResponse();

    if (myELM327.status == ELM_SUCCESS)
    {
      burn = tempBurn;
      Serial.print("DPF burns: "); Serial.println(burn);
    }
    else
      printError();
  }

// reading ends, now display it all / blink screen

  display.clear();
  display.setTextAlignment(TEXT_ALIGN_LEFT);
  display.setFont(ArialMT_Plain_16);

// main display thing

    if (burn > 0 && burn <11) {
      dpfBlink(soot, burn);
    } else {
      oneScreen(soot, km, burn, litersLeft, rpm);
      // display refresh rate is here
      delay(700);
    }
  
  display.clear();


  // check if it's still connected after the loop
  if (!myELM327.connected)
  {
    Serial.println("Lost connection..");   
      display.clear();
      display.setFont(ArialMT_Plain_16);
      display.drawString(0, 6, "Lost connection..");
      display.display();
    
    // try to reconnect
    ELM_PORT.connect("V-LINK");    
    Serial.println("reconnecting..");
      display.clear();
      display.drawString(0, 6, "reconnecting..");
      display.display(); 
    delay(500);
    
    myELM327.begin(ELM_PORT);    
    // ESP.restart();
  }  
  
}


void printError()
{
  if (myELM327.status == ELM_SUCCESS) 
    Serial.println(F("\tELM_SUCCESS"));
  else if (myELM327.status == ELM_NO_RESPONSE)
    Serial.println(F("\tERROR: ELM_NO_RESPONSE"));
  else if (myELM327.status == ELM_BUFFER_OVERFLOW)
    Serial.println(F("\tERROR: ELM_BUFFER_OVERFLOW"));
  else if (myELM327.status == ELM_GARBAGE)
    Serial.println(F("\tERROR: ELM_GARBAGE"));
  else if (myELM327.status == ELM_UNABLE_TO_CONNECT)
    Serial.println(F("\tERROR: ELM_UNABLE_TO_CONNECT"));
  else if (myELM327.status == ELM_NO_DATA)
    Serial.println(F("\tERROR: ELM_NO_DATA"));
  else if (myELM327.status == ELM_STOPPED)
    Serial.println(F("\tERROR: ELM_STOPPED"));
  else if (myELM327.status == ELM_TIMEOUT)
    Serial.println(F("\tERROR: ELM_TIMEOUT"));
  else if (myELM327.status == ELM_GENERAL_ERROR)
    Serial.println(F("\tERROR: ELM_GENERAL_ERROR")); 

  delay(200);
}

void dpfBlink(uint8_t dpfSoot,uint8_t dpfBurn)
{
        // first screen
      // Serial.println("aaah! DPF burns!");
      display.clear();
      display.setFont(ArialMT_Plain_24);
      display.drawString(5, 6, "DPF burns!");
      display.setFont(ArialMT_Plain_10);
      display.drawString(10, 38, "Don't turn off the engine");
      display.drawString(40, 52, "Soot : ");
      display.drawString(70, 52, String(dpfSoot));
      display.drawString(86, 52, "%");
      // dpfburn value
      display.setFont(ArialMT_Plain_10);
      display.drawStringMaxWidth(122, 53, 10, String(dpfBurn)); 
      display.display();
      
      // 25 * 400ms = 10 seconds, screen blinking 
      for (int a = 0; a < 12; ++a) {
      display.invertDisplay();
      delay(200);
      display.normalDisplay();
      delay(200);
      }

        // second screen
      // Serial.println("aaah! DPF burns!");
      display.clear();
      display.setFont(ArialMT_Plain_16);
      display.drawString(10, 6, "DPF soot level");
      display.setFont(ArialMT_Plain_24);
      display.drawString(45, 28, String(dpfSoot));
      display.drawString(70, 28, "%");
      // dpfburn value (0 or 6-7)
      display.setFont(ArialMT_Plain_10);
      display.drawStringMaxWidth(122, 53, 10, String(dpfBurn));       
      display.display();
      
        // screen blink
      for (int a = 0; a < 13; ++a) {
      display.invertDisplay();
      delay(200);
      display.normalDisplay();
      delay(200);      
      
      }   
    // Serial.println("DPF blink ended");
}

void oneScreen (uint8_t dpfSoot, uint16_t dpfKm, uint8_t dpfBurn, uint8_t fuel, uint16_t rpm){

      // display incorrect readings as 0
      if (dpfSoot > 130) {dpfSoot = 0;}
      if (dpfKm > 999) {dpfKm = 0;}
      if (dpfBurn > 10) {dpfBurn = 0;}
      if (fuel > 75) {fuel = 0;}
      if (rpm > 8000) {rpm = 0;}
      
      display.clear();
      display.setFont(ArialMT_Plain_16);
      display.drawStringMaxWidth(2, 0, 90, "DPF soot : ");
      display.drawStringMaxWidth(90, 0, 25, String(dpfSoot));
      display.drawStringMaxWidth(2, 18, 90, "DPF km   : ");
      display.drawStringMaxWidth(90, 18, 40, String(dpfKm));     
      display.drawStringMaxWidth(2, 36, 90, "fuel:");
      display.drawStringMaxWidth(34, 36, 40, String(fuel));         
      display.drawStringMaxWidth(56, 36, 90, "rpm:");
      display.drawStringMaxWidth(90, 36, 40, String(rpm));

      // at bottom right corner show small burn value (usually 0, burn = 6-7)
      display.setFont(ArialMT_Plain_10);
      display.drawStringMaxWidth(122, 53, 10, String(dpfBurn));               
      display.display();
}
